package net.lensfrex.socket;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashSet;

public class Server {
    private static boolean stop = false;

    public void start() {
        Server.stop = false;

        try (ServerSocket serverSocket = new ServerSocket(2333)) {
            while (!stop) {
                // Start a new thread to service more than one client
                new ServerThread(serverSocket.accept()).start();
            }
        } catch (Exception e) {
            System.err.println("Some exception happened: " + e);
        }
    }

    public void stop() {
        Server.stop = true;
    }
}

class ServerThread extends Thread {
    private static final HashSet<User> onlineUsers = new HashSet();

    private final Socket socket;

    public ServerThread(Socket socket) {
        this.socket = socket;
    }

    @Override
    public void run() {
        try (InputStream inputStream = socket.getInputStream(); OutputStream outputStream = socket.getOutputStream()) {
            byte[] clientMagicHeader = new byte[Constant.MAGIC_HEADER.length];
            if (inputStream.read(clientMagicHeader) == -1) {
                return;
            }

            if (!Arrays.equals(clientMagicHeader, Constant.MAGIC_HEADER)) {
                return;
            }

            byte[] clientOpcode = new byte[Constant.OPCODE_LENGTH];
            if (inputStream.read(clientOpcode) == -1) {
                return;
            }

            int dataLength = getLengthField(inputStream);
            if (dataLength == -1) {
                return;
            }

            this.executeAction(clientOpcode, dataLength, inputStream, outputStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void executeAction(byte[] opcode, int dataLength, InputStream inputStream, OutputStream outputStream) throws IOException {
        byte[] data = this.readRequest(dataLength, inputStream);

        if (Arrays.equals(Constant.OPCODE_ONLINE, opcode)) {
            this.login(new String(data));
            this.sendResponse(Constant.MESSAGE_OK, outputStream);
        } else if (Arrays.equals(Constant.OPCODE_OFFLINE, opcode)) {
            this.logout(new String(data));
        } else if (Arrays.equals(Constant.OPCODE_REQUEST_MESSAGE, opcode)) {

        }
    }

    private void login(String id) {
        User user = new User(id);
        onlineUsers.add(user);

        System.out.printf("User '%s' login.%n", user.getId());
    }

    private void logout(String id) {
        User user = new User(id);
        onlineUsers.add(user);
        System.out.printf("User '%s' logout.%n", user.getId());
    }

    private void

    private int sendMessage(User user, OutputStream outputStream) throws IOException {
        if (!onlineUsers.contains(user)) {

        }

        int responseLength = this.sendResponse(Util.randomString(128), outputStream);

        return responseLength;
    }

    private int getLengthField(InputStream inputStream) throws IOException {
        byte[] tmp = new byte[4];
        if (inputStream.read(tmp) == -1) {
            return -1;
        }

        return Util.bytesToInt32(tmp);
    }

    private void receiveMessage(DataPack dataPack) {
        byte[] data = dataPack.getData();

        User user = new User();

        String message = new String(data);
        System.out.printf("Receive message '%s' from: '%s'%n", user.getId(), message);
        System.out.println("----------------");
    }

    private DataPack readRequest(InputStream inputStream) throws IOException {
        byte[] clientOpcode = new byte[Constant.OPCODE_LENGTH];
        if (inputStream.read(clientOpcode) == -1) {
            return null;
        }

        int dataLength = this.getLengthField(inputStream);

        byte[] data = new byte[dataLength];
        inputStream.read(data);

        return new DataPack(clientOpcode, data);
    }

    private int sendResponse(String message, OutputStream outputStream) throws IOException {
        return this.sendResponse(message.getBytes(StandardCharsets.UTF_8), outputStream);
    }

    private int sendResponse(byte[] data, OutputStream outputStream) throws IOException {
        DataPack dataPack = new DataPack(Constant.OPCODE_SEND_MESSAGE, data);

        return this.sendResponse(dataPack, outputStream);
    }

    private int sendResponse(DataPack dataPack, OutputStream outputStream) throws IOException {
        outputStream.write(Constant.MAGIC_HEADER);
        outputStream.write(dataPack.getOpcode());

        outputStream.write(Util.int32ToBytes(dataPack.getDataLength()));
        outputStream.write(dataPack.getData());

        return dataPack.getDataLength();
    }
}