package net.lensfrex.socket;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Random;

public class Util {
    private static final char[] alphabet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789".toCharArray();
    private static final Random random = new Random();

    public static String randomString(int length) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            sb.append(alphabet[random.nextInt(alphabet.length)]);
        }

        return sb.toString();
    }

    public static int bytesToInt32(byte[] bytes) {
        int result = 0;
        for (int i = 0; i < 4; i++) {
            int shift = (3 - i) * 8;
            result += (bytes[i] & 0xFF) << shift;
        }

        return result;
    }

    public static byte[] int32ToBytes(int i) {
        byte[] result = new byte[4];
        result[0] = (byte) ((i >> 24) & 0xFF);
        result[1] = (byte) ((i >> 16) & 0xFF);
        result[2] = (byte) ((i >> 8) & 0xFF);
        result[3] = (byte) (i & 0xFF);

        return result;
    }

    public static byte[] readAllData(InputStream inputStream) {
        return readAllData(inputStream, 5);// read every 5kb in default
    }

    public static byte[] readAllData(InputStream inputStream, int byteAllocation) {
//        On Java 9 and newer version, can be more simply:
//        try (ByteArrayOutputStream byteArrayInputStream = new ByteArrayOutputStream(); inputStream) {

        try (ByteArrayOutputStream byteArrayInputStream = new ByteArrayOutputStream()) {
            byte[] bytes = new byte[1024 * byteAllocation];

            for (int length; (length = inputStream.read(bytes)) != -1; ) {
                byteArrayInputStream.write(bytes, 0, length);
            }

            byteArrayInputStream.flush();

            return byteArrayInputStream.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            try {
                inputStream.close();
            } catch (IOException ignored) {
            }
        }
    }
}
